use gtk4 as gtk;

use gtk::prelude::*;
use super::switch::*;

pub struct EventHandler<F, C>{
    component: C,
    callback: F,
}

impl<F, C> EventHandler<F, C>
where F: Fn(&C) + FnOnce(&C) + FnMut(&C){
    pub fn new(component: C, callback: F) -> EventHandler<F, C>{
        Self{
            component,
            callback,
        }
    }
}

pub trait BtnEventHandler{
    fn on_click(self) -> ();
}

pub trait SwEventHandler{
    fn on_toggle(self) -> ();
}

impl<F, C> BtnEventHandler for EventHandler<F, C>
    where F: Fn(&C) + FnOnce(&C) + FnMut(&C) + 'static, C: ButtonExt + WidgetExt{
    fn on_click(self) -> () {
        self.component.connect_clicked(move |button| {
            (self.callback)(button)
        });
    }
}

impl<F, C> SwEventHandler for EventHandler<F, C>
    where F: Fn(&C) + FnOnce(&C) + FnMut(&C) + 'static, C: SwitchExt + WidgetExt{
    fn on_toggle(self) -> () {
        self.component.connect_state_notify(move |switch| {
            (self.callback)(switch)
        });
    }
}


